package ru.alphatech.icfpc.policy;

import java.util.ArrayList;

import ru.alphatech.icfpc.cards.Copy;
import ru.alphatech.icfpc.cards.Dec;
import ru.alphatech.icfpc.cards.Get;
import ru.alphatech.icfpc.cards.ICard;
import ru.alphatech.icfpc.cards.Inc;
import ru.alphatech.icfpc.cards.K;
import ru.alphatech.icfpc.cards.Put;
import ru.alphatech.icfpc.cards.Revive;
import ru.alphatech.icfpc.cards.S;
import ru.alphatech.icfpc.cards.Zero;
import ru.alphatech.icfpc.game.AbstractPolicy;
import ru.alphatech.icfpc.game.Batch;
import ru.alphatech.icfpc.game.IValue;
import ru.alphatech.icfpc.game.Run;
import ru.alphatech.icfpc.game.Slot;
import ru.alphatech.icfpc.game.Value;
import ru.alphatech.icfpc.game.Batch.OperationType;

public class OneArgumentCardsPolicy extends AbstractPolicy{
	
	public enum TYPE {
		DEC,
		INC,
		COPY,
		REVIVE
	}
	
	private TYPE type;
	private int processingSlotIndex = -1;
	private int registerIndex = -1;
	private int targetSlotIndex = -1;
	
	
	public OneArgumentCardsPolicy(TYPE type, int processingSlot, int target, int registerIndex) {
		this.type = type;
		this.processingSlotIndex = processingSlot;
		this.targetSlotIndex = target;
		this.registerIndex = registerIndex;
	}
	
	public ICard getCard(){
		switch (type) {
			case DEC:
			return new Dec();
			case INC:
			return new Inc();
			case COPY:
			return new Copy();
			case REVIVE:
			return new Revive();
		}
		return null;
	}
	
	protected boolean isEqualsCard(IValue value){
		if(value instanceof IValue)
			return false;
		if( type == TYPE.DEC && value instanceof Dec)
			return true;
		if( type == TYPE.INC && value instanceof Inc)
			return true;
		if( type == TYPE.COPY && value instanceof Copy)
			return true;
		if( type == TYPE.REVIVE && value instanceof Revive)
			return true;
		return false;
	}

	@Override
	public ArrayList<Batch> getBatches() {
		ArrayList<Batch> retArray = new ArrayList<Batch>();
		
		retArray.addAll(numberIntoSlotField(Run.getRegisterByIndex(registerIndex), targetSlotIndex));
		
		Slot processingSlot = Run.getProponent().getSlot(processingSlotIndex);
		IValue slotValue = processingSlot.getValue();
		
		if(slotValue instanceof Value){
			retArray.add(new Batch(OperationType.CARD_TO_SLOT, new Put(), processingSlot));
			retArray.add(new Batch(OperationType.SLOT_TO_CARD, getCard(), processingSlot));
		}else{
			ICard currentCard = (ICard) slotValue;
			if(!(currentCard instanceof Dec && type == TYPE.DEC) || 
			   !(currentCard instanceof Inc && type == TYPE.INC))
			{
				retArray.add(new Batch(OperationType.CARD_TO_SLOT, new Put(), processingSlot));
				retArray.add(new Batch(OperationType.SLOT_TO_CARD, getCard(), processingSlot));
			}
		}
		
		retArray.add(new Batch(OperationType.CARD_TO_SLOT, new K(), processingSlot));
		retArray.add(new Batch(OperationType.CARD_TO_SLOT, new S(), processingSlot));
		
		retArray.add(new Batch(OperationType.SLOT_TO_CARD, new Get(), processingSlot));
		retArray.add(new Batch(OperationType.SLOT_TO_CARD, new Zero(), processingSlot));
		
		return retArray;
	}
	
}
